<?php

namespace Drupal\webform_password_protection_handler;

use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Define class to provide token service.
 */
class Token {

  /**
   * Request.
   *
   * @var null|\Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Constructs a new Token object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack service.
   */
  public function __construct(
    RequestStack $request_stack,
  ) {
    $this->request = $request_stack->getMainRequest();
  }

  /**
   * Return token && jwt token.
   *
   * @param mixed $data
   *   Data to encode.
   * @param mixed $key
   *   Key to encode.
   * @param array $opts
   *   Host: set token iss and aud.
   */
  public function encode($data, $key, array $opts = []) {
    $iat = new \DateTime('now', new \DateTimeZone('UTC'));
    $exp = new \DateTime();
    $host = $opts['host'] ?? $this->request->getHost();
    // P mandatory for specifying period
    // T required if time interval include time element
    // H hours
    // Here session works for 1 hours.
    $exp->add(new \DateInterval('PT1H'));
    $exp->setTimezone(new \DateTimeZone('UTC'));
    $token = [
      "iss" => $host,
      "aud" => $host,
      "iat" => $iat->getTimestamp(),
      "nbf" => $iat->getTimestamp(),
      "exp" => $exp->getTimestamp(),
      "data" => $data,
    ];
    return [$token, JWT::encode($token, $key, 'HS256')];
  }

  /**
   * FirebaseJWT handle hmac tampering test himself.
   */
  public function decode($jwt, $key) {
    try {
      $decoded = JWT::decode($jwt, new Key($key, 'HS256'));
      $decoded_array = (array) $decoded;
      return $decoded_array;
    }
    catch (\Exception $e) {
      return NULL;
    }
  }

}
